@@ -26,15 +26,22 @@ _downloadChunkSize = 4096
 class ElevenLabsVoice:
     """
     Represents a voice in the ElevenLabs API.
-    It's the parent class for all voices, and used directly for the premade ones.
+
+    It's the parent class for all voices, and used directly for the premade and designed ones.
     """
     @staticmethod
     def edit_stream_settings(playbackBlockSize=None, downloadChunkSize=None) -> None:
         """
-        This function lets you override the values used for the streaming function FOR ALL VOICES.
-        Please only do this if you know what you're doing.
-        :param playbackBlockSize: The size (in bytes) of the byte blocks used for playback.
-        :param downloadChunkSize: The size (in bytes) of the chunks to be downloaded.
+        This function lets you override the default values used for the streaming function.
+
+        Danger:
+            This change affects all voices.
+
+            Please only do this if you know what you're doing.
+
+        Parameters:
+            playbackBlockSize (int): The size (in frames) of the blocks used for playback.
+            downloadChunkSize (int): The size (in bytes) of the chunks to be downloaded.
         """
         global _playbackBlockSize, _downloadChunkSize
         if playbackBlockSize is not None:
@@ -43,19 +50,23 @@ class ElevenLabsVoice:
             _downloadChunkSize = downloadChunkSize
 
     @staticmethod
-    def voiceFactory(voiceData, linkedUser: ElevenLabsUser) -> ElevenLabsVoice | ElevenLabsGeneratedVoice | ElevenLabsClonedVoice:
+    def voiceFactory(voiceData, linkedUser: ElevenLabsUser) -> ElevenLabsVoice | ElevenLabsDesignedVoice | ElevenLabsClonedVoice:
         """
-        Initializes a new instance of either ElevenLabsVoice or ElevenLabsGeneratedVoice or ElevenLabsClonedVoice depending on the category.
+        Initializes a new instance of ElevenLabsVoice or one of its subclasses depending on voiceData.
+
         Args:
             voiceData: A dictionary containing the voice data.
             linkedUser: An instance of the ElevenLabsUser class representing the linked user.
+
+        Returns:
+            ElevenLabsVoice | ElevenLabsDesignedVoice | ElevenLabsClonedVoice: The voice object
         """
         if voiceData["category"] == "premade":
             return ElevenLabsVoice(voiceData, linkedUser)
         elif voiceData["category"] == "cloned":
             return ElevenLabsClonedVoice(voiceData, linkedUser)
         elif voiceData["category"] == "generated":
-            return ElevenLabsGeneratedVoice(voiceData, linkedUser)
+            return ElevenLabsDesignedVoice(voiceData, linkedUser)
         else:
             raise ValueError(voiceData["category"] + " is not a valid voice category!")
 
@@ -63,6 +74,7 @@ class ElevenLabsVoice:
         """
         Initializes a new instance of the ElevenLabsVoice class.
         Don't use this constructor directly. Use the factory instead.
+
         Args:
             voiceData: A dictionary containing the voice data.
             linkedUser: An instance of the ElevenLabsUser class representing the linked user.
@@ -73,7 +85,64 @@ class ElevenLabsVoice:
         self.initialName = voiceData["name"]
         self._voiceID = voiceData["voice_id"]
         self._category = voiceData["category"]
-        self._labels = voiceData["labels"]
+
+    def get_settings(self) -> dict:
+        """
+        Returns:
+            dict: The current generation settings of the voice (stability and clarity).
+        """
+        response = _api_get("/voices/" + self._voiceID + "/settings", self._linkedUser.headers)
+        return response.json()
+    def get_info(self) -> dict:
+        """
+        Tip:
+            I've only added specific getters for the most common attributes (name/description).
+
+            Use this function for all other metadata.
+
+        Returns:
+            dict: A dict containing all the metadata for the voice, such as the name, the description, etc.
+        """
+        response = _api_get("/voices/" + self._voiceID, self._linkedUser.headers)
+        return response.json()
+
+    def get_name(self) -> str:
+        """
+        Returns:
+            str: The name of the voice.
+        """
+        return self.get_info()["name"]
+
+    def get_description(self) -> str|None:
+        """
+        Returns:
+            str: The description of the voice.
+        """
+        return self.get_info()["description"]
+
+    def edit_settings(self, stability:float=None, similarity_boost:float=None):
+        """
+        Note:
+            If either argument is omitted, the current values will be used instead.
+
+        Edit the settings of the current voice.
+
+        Args:
+            stability (float, optional): The stability of the voice.
+            similarity_boost (float, optional): The similarity boost of the voice.
+
+        Raises:
+            ValueError: If the provided stability or similarity_boost value is not between 0 and 1.
+        """
+        if stability is None or similarity_boost is None:
+            oldSettings = self.get_settings()
+            if stability is None: stability = oldSettings["stability"]
+            if similarity_boost is None: stability = oldSettings["similarity_boost"]
+
+        if not(0 <= stability <= 1 and 0 <= similarity_boost <= 1):
+            raise ValueError("Please provide a value between 0 and 1.")
+        payload = {"stability": stability, "similarity_boost": similarity_boost}
+        _api_json("/voices/" + self._voiceID + "/settings/edit", self._linkedUser.headers, jsonData=payload)
 
     def _generate_payload(self, prompt:str, stability:Optional[float]=None, similarity_boost:Optional[float]=None) -> dict:
         """
@@ -103,16 +172,18 @@ class ElevenLabsVoice:
         """
         Generates speech for the given prompt and returns the audio data as bytes of an mp3 file.
 
+        Tip:
+            If you would like to save the audio to disk or otherwise, you can use helpers.save_audio_bytes().
+
         Args:
         	prompt: The prompt to generate speech for.
         	stability: A float between 0 and 1 representing the stability of the generated audio. If None, the current stability setting is used.
         	similarity_boost: A float between 0 and 1 representing the similarity boost of the generated audio. If None, the current similarity boost setting is used.
 
         Returns:
-        	The bytes of the audio file, and the json data (such as the number of tokens used).
+        	The bytes of the audio file.
+
         """
-        #The output from the site is an mp3 file.
-        #You can check the README for an example of how to convert it to wav on the fly using pydub and bytesIO.
         payload = self._generate_payload(prompt, stability, similarity_boost)
         response = _api_json("/text-to-speech/" + self._voiceID + "/stream", self._linkedUser.headers, jsonData=payload)
 
@@ -125,6 +196,10 @@ class ElevenLabsVoice:
         """
         Generate audio bytes from the given prompt and play them using sounddevice.
 
+        Tip:
+            This function downloads the entire file before playing it back, and even if playInBackground is set, it will halt execution until the file is downloaded.
+            If you need faster response times and background downloading and playback, use generate_and_stream_audio.
+
         Parameters:
         	prompt (str): The text prompt to generate audio from.
         	playInBackground (bool): Whether to play audio in the background or wait for it to finish playing.
@@ -133,40 +208,41 @@ class ElevenLabsVoice:
         	similarity_boost: A float between 0 and 1 representing the similarity boost of the generated audio. If None, the current similarity boost setting is used.
         	onPlaybackStart: Function to call once the playback begins
         	onPlaybackEnd: Function to call once the playback ends
+
         Returns:
-        The data from the response (such as the number of tokens used).
+            The audio bytes.
         """
         audioData = self.generate_audio_bytes(prompt, stability, similarity_boost)
         play_audio_bytes(audioData, playInBackground, portaudioDeviceID, onPlaybackStart, onPlaybackEnd)
-        return
+        return audioData
 
     def generate_and_stream_audio(self, prompt:str, portaudioDeviceID:Optional[int] = None,
                                   stability:Optional[float]=None, similarity_boost:Optional[float]=None, streamInBackground=False,
                      onPlaybackStart:Callable=lambda: None, onPlaybackEnd:Callable=lambda: None):
         """
-        Generate audio bytes from the given prompt and play them using sounddevice in callback mode.
-        Sometimes make the audio skip slightly, but the audio begins playing more quickly.
-        I've measured it and on average it takes around 0.5-ish seconds less than the normal generate function.
-        It may be less depending on your subscription level.
+
+        Note:
+            No longer suffers from the skipping issues it had in the past.
+
+        Generate audio bytes from the given prompt and stream them using sounddevice.
+
+        If streamInBackground is true, it will download the audio data in a separate thread, without pausing the main thread.
+
         Parameters:
             streamInBackground (bool): Whether or not to play the audio (and let the download complete) in a separate thread.
-        	prompt (str): The text prompt to generate audio from.
-        	portaudioDeviceID (int, optional): The ID of the audio device to use for playback. Defaults to the default output device.
-        	stability: A float between 0 and 1 representing the stability of the generated audio. If None, the current stability setting is used.
-        	similarity_boost: A float between 0 and 1 representing the similarity boost of the generated audio. If None, the current similarity boost setting is used.
-        	onPlaybackStart: Function to call once the playback begins
-        	onPlaybackEnd: Function to call once the playback ends
-        Returns:
-        None
+            prompt (str): The text prompt to generate audio from.
+            portaudioDeviceID (int, optional): The ID of the audio device to use for playback. Defaults to the default output device.
+            stability: A float between 0 and 1 representing the stability of the generated audio. If None, the current stability setting is used.
+            similarity_boost: A float between 0 and 1 representing the similarity boost of the generated audio. If None, the current similarity boost setting is used.
+            onPlaybackStart: Function to call once the playback begins
+            onPlaybackEnd: Function to call once the playback ends
+
         """
         payload = self._generate_payload(prompt, stability, similarity_boost)
         path = "/text-to-speech/" + self._voiceID + "/stream"
 
         streamedResponse = requests.post(api_endpoint + path, headers=self._linkedUser.headers, json=payload, stream=True)
 
-        #testURL = "https://litter.catbox.moe/sk8cwj.mp3"
-        #streamedResponse = requests.get(testURL, stream=True)
-
         streamer = _AudioChunkStreamer(portaudioDeviceID, onPlaybackStart, onPlaybackEnd)
 
         if streamInBackground:
@@ -177,29 +253,18 @@ class ElevenLabsVoice:
 
         return
 
-    def play_preview(self, playInBackground:bool, portaudioDeviceID:Optional[int] = None,
-                                onPlaybackStart:Callable=lambda: None, onPlaybackEnd:Callable=lambda: None) -> None:
-        """
-        Plays the preview audio.
 
-        Args:
-            playInBackground: A bool indicating whether to play the audio in the background.
-            portaudioDeviceID: Optional int indicating the device ID to use for audio playback.
-        	onPlaybackStart: Function to call once the playback begins
-        	onPlaybackEnd: Function to call once the playback ends
-        Returns:
-            None
+    def get_preview_url(self) -> str|None:
         """
-        # This will error out if the preview hasn't been generated
-        play_audio_bytes(self.get_preview_bytes(), playInBackground, portaudioDeviceID, onPlaybackStart, onPlaybackEnd)
-        return
+        Returns:
+            str|None: The preview URL of the voice, or None if it hasn't been generated.
+        """
+        return self.get_info()["preview_url"]
 
     def get_preview_bytes(self) -> bytes:
         """
-        Returns the preview audio in bytes.
-
         Returns:
-            bytes: The preview audio in bytes.
+            bytes: The preview audio bytes.
 
         Raises:
             RuntimeError: If no preview URL is available.
@@ -210,70 +275,40 @@ class ElevenLabsVoice:
             raise RuntimeError("No preview URL available!")
         response = requests.get(previewURL, allow_redirects=True)
         return response.content
-    def get_settings(self) -> dict:
-        """
-        Get the name of the current voice.
 
-        Returns:
-            str: The name of the voice.
+    def play_preview(self, playInBackground:bool, portaudioDeviceID:Optional[int] = None,
+                                onPlaybackStart:Callable=lambda: None, onPlaybackEnd:Callable=lambda: None) -> None:
         """
-        # We don't store the name OR the settings, as they can be changed externally.
-        response = _api_get("/voices/" + self._voiceID + "/settings", self._linkedUser.headers)
-        return response.json()
-
-    def get_name(self) -> str:
-        """
-        Get the name of the current voice.
-
-        Returns:
-            str: The name of the voice.
-        """
-        response = _api_get("/voices/" + self._voiceID, self._linkedUser.headers)
-        return response.json()["name"]
-
-    def get_preview_url(self) -> str|None:
-        """
-        Get the preview URL of the current voice.
-
-        Returns:
-            str|None: The preview URL of the voice, or None if it doesn't exist.
-        """
-        response = _api_get("/voices/" + self._voiceID, self._linkedUser.headers)
-        return response.json()["preview_url"]
-
-    def edit_settings(self, stability:float=None, similarity_boost:float=None):
-        """
-        Edit the settings of the current voice.
+        Plays the preview audio.
 
         Args:
-            stability (float, optional): The stability of the voice. If None, the current stability setting will be used. Defaults to None.
-            similarity_boost (float, optional): The similarity boost of the voice. If None, the current similarity boost setting will be used. Defaults to None.
+            playInBackground: A bool indicating whether to play the audio in the background.
+            portaudioDeviceID: Optional int indicating the device ID to use for audio playback.
+        	onPlaybackStart: Function to call once the playback begins
+        	onPlaybackEnd: Function to call once the playback ends
 
-        Raises:
-            ValueError: If the provided stability or similarity_boost value is not between 0 and 1.
+        Returns:
+            None
         """
-        if stability is None or similarity_boost is None:
-            oldSettings = self.get_settings()
-            if stability is None: stability = oldSettings["stability"]
-            if similarity_boost is None: stability = oldSettings["similarity_boost"]
 
-        if not(0 <= stability <= 1 and 0 <= similarity_boost <= 1):
-            raise ValueError("Please provide a value between 0 and 1.")
-        payload = {"stability": stability, "similarity_boost": similarity_boost}
-        _api_json("/voices/" + self._voiceID + "/settings/edit", self._linkedUser.headers, jsonData=payload)
+        play_audio_bytes(self.get_preview_bytes(), playInBackground, portaudioDeviceID, onPlaybackStart, onPlaybackEnd)
+        return
 
     @property
     def category(self):
+        """
+        This property indicates the "type" of the voice, whether it's premade, cloned, designed etc.
+        """
         return self._category
-    @property
-    def labels(self):
-        return self._labels
 
-    # Since the same voice can be available for multiple users, we allow the user to change which API key is used.
     @property
     def linkedUser(self):
         """
-        Returns the user currently linked to the voice, whose API key will be used.
+        Note:
+            This property can also be set.
+            This is mostly in case some future update adds shared voices (beyond the currently available premade ones).
+
+        The user currently linked to the voice, whose API key will be used to generate audio.
 
         Returns:
             ElevenLabsUser: The user linked to the voice.
@@ -289,9 +324,6 @@ class ElevenLabsVoice:
         Args:
             newUser (ElevenLabsUser): The new user to link to the voice.
 
-        Returns:
-            None
-
         """
         self._linkedUser = newUser
 
@@ -300,7 +332,105 @@ class ElevenLabsVoice:
         return self._voiceID
 
 
+class ElevenLabsDesignedVoice(ElevenLabsVoice):
+    def __init__(self, voiceData, linkedUser: ElevenLabsUser):
+        super().__init__(voiceData, linkedUser)
 
+    def edit_voice(self, newName:str = None, newLabels:dict[str, str] = None, description:str = None):
+        """
+        Edit the name/labels of the voice.
+
+        Args:
+            newName (str): The new name
+            newLabels (str): The new labels
+            description (str): The new description
+        """
+        currentInfo = self.get_info()
+        payload = {
+            "name": currentInfo["name"],
+            "labels": currentInfo["labels"],
+            "description": currentInfo["description"]
+        }
+        if newName is not None:
+            payload["name"] = newName
+        if newLabels is not None:
+            if len(newLabels.keys()) > 5:
+                raise ValueError("Too many labels! The maximum amount is 5.")
+            payload["labels"] = newLabels
+        if description is not None:
+            payload["description"] = description
+        _api_multipart("/voices/" + self._voiceID + "/edit", self._linkedUser.headers, data=payload)
+    def delete_voice(self):
+        """
+        This function deletes the voice, and also sets the voiceID to be empty.
+        """
+        if self._category == "premade":
+            raise RuntimeError("Cannot delete premade voices!")
+        response = _api_del("/voices/" + self._voiceID, self._linkedUser.headers)
+        self._voiceID = ""
+
+
+class ElevenLabsClonedVoice(ElevenLabsDesignedVoice):
+    def __init__(self, voiceData, linkedUser: ElevenLabsUser):
+        super().__init__(voiceData, linkedUser)
+
+    def get_samples(self) -> list[ElevenLabsSample]:
+        outputList = list()
+        samplesData = self.get_info()["samples"]
+        from elevenlabslib.ElevenLabsSample import ElevenLabsSample
+        for sampleData in samplesData:
+            outputList.append(ElevenLabsSample(sampleData, self))
+        return outputList
+
+    def add_samples_by_path(self, samples:list[str]|str):
+        """
+        This function adds samples to the current voice by their file paths.
+
+        Args:
+            samples (list[str]|str): A list with the file paths to the audio files or a str containing a single path.
+
+        Raises:
+            ValueError: If no samples are provided.
+
+        """
+        if isinstance(samples, str):
+            samples = list(samples)
+
+        sampleBytes = {}
+        for samplePath in samples:
+            if "\\" in samplePath:
+                fileName = samplePath[samplePath.rindex("\\")+1:]
+            else:
+                fileName = samplePath
+            sampleBytes[fileName] = open(samplePath, "rb").read()
+        self.add_samples_bytes(sampleBytes)
+
+    #Requires a dict of filenames and bytes
+    def add_samples_bytes(self, samples:dict[str, bytes]):
+        """
+        This function adds samples to the current voice by their file names and bytes.
+
+        Args:
+            samples (dict[str, bytes]): A dictionary of audio file names and their respective bytes.
+
+        Raises:
+            ValueError: If no samples are provided.
+
+        """
+        if len(samples.keys()) == 0:
+            raise ValueError("Please add at least one sample!")
+
+        payload = {"name":self.get_name()}
+        files = list()
+        for fileName, fileBytes in samples.items():
+            files.append(("files", (fileName, io.BytesIO(fileBytes))))
+
+        _api_multipart("/voices/" + self._voiceID + "/edit", self._linkedUser.headers, data=payload, filesData=files)
+
+
+
+#This way lies only madness.
+_defaultDType = "float32"
 class _AudioChunkStreamer:
     def __init__(self,portaudioDeviceID:int = None,onPlaybackStart:Callable=lambda: None, onPlaybackEnd:Callable=lambda: None):
         self._q = queue.Queue()
@@ -309,6 +439,7 @@ class _AudioChunkStreamer:
         self._bytesLock = threading.Lock()
         self._onPlaybackStart = onPlaybackStart
         self._onPlaybackEnd = onPlaybackEnd
+        self._frameSize = 0
 
         if portaudioDeviceID is None:
             portaudioDeviceID = sd.default.device
@@ -325,8 +456,6 @@ class _AudioChunkStreamer:
         }
 
     def begin_streaming(self, streamedResponse:requests.Response):
-        #After re-relooking at this, I was stupid, the queue is actually necessary.
-
         # Clean all the buffers and reset all events.
         self._q = queue.Queue()
         self._bytesFile = io.BytesIO()
@@ -345,6 +474,7 @@ class _AudioChunkStreamer:
                 with self._bytesLock:
                     self._bytesSoundFile = sf.SoundFile(self._bytesFile)
                     logging.debug("File created (" + str(self._bytesFile.tell()) + " bytes read).")
+                    self._frameSize = self._bytesSoundFile.channels * sf._ffi.sizeof(self._bytesSoundFile._check_dtype(_defaultDType))
                     self._events["soundFileReadyEvent"].set()
                     break
             except sf.LibsndfileError:
@@ -357,24 +487,14 @@ class _AudioChunkStreamer:
 
         stream = sd.RawOutputStream(
             samplerate=self._bytesSoundFile.samplerate, blocksize=_playbackBlockSize,
-            device=self._deviceID, channels=self._bytesSoundFile.channels, dtype='float32',
+            device=self._deviceID, channels=self._bytesSoundFile.channels, dtype=_defaultDType,
             callback=self._stream_playback_callback, finished_callback=self._events["playbackFinishedEvent"].set)
         logging.debug("Starting playback...")
         with stream:
-            #This timeout is actually irrelevant now because I'm making the queue infinite size.
-            #timeout = _playbackBlockSize * _playbackBufferSizeInBlocks / self._bytesSoundFile.samplerate
-
-            # Since I can't find any way to get the buffer size from soundfile,
-            # we will just assume the first read operation gives back a complete chunk
-            # and use that to check later reads. This SHOULD be accurate. Hopefully.
-            # Maybe there's like a weird edge case or something, hopefully not.
-            likelyReadChunkSize = -1
             while True:
-                data = self._insert_into_queue_from_download_thread()
-                if likelyReadChunkSize == -1:
-                    likelyReadChunkSize = len(data)
-                if len(data) >= likelyReadChunkSize:
-                    #logging.debug("Putting " + str(len(data)) + " bytes in queue.")
+                data = self._get_data_from_download_thread()
+                if len(data) == _playbackBlockSize*self._frameSize:
+                    logging.debug("Putting " + str(len(data)) + " bytes in queue.")
                     self._q.put(data)
                 else:
                     logging.debug("Got back less data than expected, check if we're at the end...")
@@ -392,7 +512,7 @@ class _AudioChunkStreamer:
                                 self._q.put(data)
                             break
                         else:
-                            print("We're not at the end. Wait for more data.")
+                            logging.critical("We're not at the end, yet we recieved less data than expected. THIS SHOULD NOT HAPPEN.")
             logging.debug("While loop done.")
             self._events["playbackFinishedEvent"].wait()  # Wait until playback is finished
             self._onPlaybackEnd()
@@ -443,12 +563,6 @@ class _AudioChunkStreamer:
 
     def _stream_playback_callback(self, outdata, frames, timeData, status):
         assert frames == _playbackBlockSize
-        if status.output_underflow:
-            logging.error('Output underflow: increase blocksize?')
-            #raise sd.CallbackAbort
-        #assert not status
-
-        #Underflow can happen when two streams are started at once and it doesn't seem to do anything bad so I'm just ignoring it.
 
         while True:
             try:
@@ -469,7 +583,7 @@ class _AudioChunkStreamer:
 
         if not self._events["playbackStartFired"].is_set(): #Ensure the callback only fires once.
             self._events["playbackStartFired"].set()
-            print("Firing onPlaybackStart...")
+            logging.debug("Firing onPlaybackStart...")
             self._onPlaybackStart()
 
         # Last read chunk was smaller than it should've been. It's either EOF or that stupid soundFile bug.
@@ -496,35 +610,38 @@ class _AudioChunkStreamer:
         else:
             outdata[:] = readData
     #THIS FUNCTION ASSUMES YOU'VE GIVEN THE THREAD THE LOCK.
-    def _soundFile_read_and_fix(self, dataToRead:int=-1, dtype="float32"):
+    def _soundFile_read_and_fix(self, dataToRead:int=-1, dtype=_defaultDType):
         readData = self._bytesSoundFile.buffer_read(dataToRead, dtype=dtype)
-        if len(readData) == 0:
-            logging.debug("No data read.")
-            logging.debug("Frame counter must be outdated, recreating soundfile...")
-            self._bytesFile.seek(0)
-            newSF = sf.SoundFile(self._bytesFile)
-            newSF.seek(self._bytesSoundFile.tell())
-            self._bytesSoundFile = newSF
-            readData = self._bytesSoundFile.buffer_read(dataToRead, dtype=dtype)
-            logging.debug("Now read " + str(len(readData)) + " bytes. I sure fucking hope that number isn't zero.")
+        logging.debug(f"Expected {dataToRead*self._frameSize} bytes, got back {len(readData)}")
+        if len(readData) < dataToRead * self._frameSize:
+            logging.debug("Insufficient data read.")
+            curPos = self._bytesFile.tell()
+            endPos = self._bytesFile.seek(0, os.SEEK_END)
+            if curPos != endPos:
+                logging.debug("We're not at the end of the file. Check if we're out of frames.")
+                logging.debug("Recreating soundfile...")
+                self._bytesFile.seek(0)
+                newSF = sf.SoundFile(self._bytesFile, mode="r")
+                newSF.seek(self._bytesSoundFile.tell() - int(len(readData) / self._frameSize))
+                if newSF.frames > self._bytesSoundFile.frames:
+                    logging.debug("Frame counter was outdated.")
+                    self._bytesSoundFile = newSF
+                    readData = self._bytesSoundFile.buffer_read(dataToRead, dtype=dtype)
+                    logging.debug("Now read " + str(len(readData)) +
+                          " bytes. I sure hope that number isn't zero.")
         return readData
 
-    def _insert_into_queue_from_download_thread(self) -> bytes:
+    def _get_data_from_download_thread(self) -> bytes:
         self._events["blockDataAvailable"].wait()  # Wait until a block of data is available.
         self._bytesLock.acquire()
         try:
             readData = self._soundFile_read_and_fix(_playbackBlockSize)
-        except AssertionError as e:
-            logging.debug("Exception in buffer_read (likely not enough data left), read what is available...")
-            try:
-                readData = self._soundFile_read_and_fix()
-            except AssertionError as en:
-                logging.debug("Mismatch in the number of frames read.")
-                logging.debug("This only seems to be an issue when it happens with files that have ID3v2 tags.")
-                logging.debug("Ignore it and return empty.")
-                readData = b""
+        except AssertionError as en:
+            logging.debug("Mismatch in the number of frames read.")
+            logging.debug("This only seems to be an issue when it happens with files that have ID3v2 tags.")
+            logging.debug("Ignore it and return empty.")
+            readData = b""
 
-        #logging.debug("Checking remaining bytes...")
         currentPos = self._bytesFile.tell()
         self._bytesFile.seek(0, os.SEEK_END)
         endPos = self._bytesFile.tell()
@@ -541,137 +658,3 @@ class _AudioChunkStreamer:
         self._bytesLock.release()
         return readData
 
-
-class ElevenLabsGeneratedVoice(ElevenLabsVoice):
-    def __init__(self, voiceData, linkedUser: ElevenLabsUser):
-        super().__init__(voiceData, linkedUser)
-
-    def edit_voice(self, newName:str = None, newLabels:dict[str, str] = None):
-        """
-        Edit the name/labels of the voice.
-
-        Args:
-            newName (str): The new name for the voice.
-            newLabels (str): The new labels for the voice.
-        """
-        if newName is None:
-            newName = self.get_name()
-        payload = {"name": newName}
-
-        if newLabels is not None:
-            if len(newLabels.keys()) > 5:
-                raise ValueError("Too many labels! The maximum amount is 5.")
-            payload["labels"] = newLabels
-        _api_multipart("/voices/" + self._voiceID + "/edit", self._linkedUser.headers, data=payload)
-    def delete_voice(self):
-        """
-        This function deletes the current voice.
-
-        Returns:
-            None
-
-        Raises:
-            RuntimeError: If the voice is a premade voice.
-
-        """
-        if self._category == "premade":
-            raise RuntimeError("Cannot delete premade voices!")
-        response = _api_del("/voices/" + self._voiceID, self._linkedUser.headers)
-        self._voiceID = ""
-
-
-class ElevenLabsClonedVoice(ElevenLabsVoice):
-    def __init__(self, voiceData, linkedUser: ElevenLabsUser):
-        super().__init__(voiceData, linkedUser)
-
-
-    def get_samples(self) -> list[ElevenLabsSample]:
-        response = _api_get("/voices/" + self._voiceID, self._linkedUser.headers)
-        outputList = list()
-        samplesData = response.json()["samples"]
-        from elevenlabslib.ElevenLabsSample import ElevenLabsSample
-        for sampleData in samplesData:
-            outputList.append(ElevenLabsSample(sampleData, self))
-        return outputList
-
-    def edit_voice(self, newName:str = None, newLabels:dict[str, str] = None):
-        """
-        Edit the name/labels of the voice.
-
-        Args:
-            newName (str): The new name for the voice.
-            newLabels (str): The new labels for the voice.
-        """
-        if newName is None:
-            newName = self.get_name()
-        payload = {"name": newName}
-
-        if newLabels is not None:
-            if len(newLabels.keys()) > 5:
-                raise ValueError("Too many labels! The maximum amount is 5.")
-            payload["labels"] = newLabels
-        _api_multipart("/voices/" + self._voiceID + "/edit", self._linkedUser.headers, data=payload)
-
-    def add_samples_by_path(self, samples:list[str]):
-        """
-        This function adds samples to the current voice by their file paths.
-
-        Args:
-            samples (list[str]): A list of file paths to the sample audio files.
-
-        Returns:
-            None
-
-        Raises:
-            ValueError: If no samples are provided.
-
-        """
-        sampleBytes = {}
-        for samplePath in samples:
-            if "\\" in samplePath:
-                fileName = samplePath[samplePath.rindex("\\")+1:]
-            else:
-                fileName = samplePath
-            sampleBytes[fileName] = open(samplePath, "rb").read()
-        self.add_samples_bytes(sampleBytes)
-
-    #Requires a dict of filenames and bytes
-    def add_samples_bytes(self, samples:dict[str, bytes]):
-        """
-        This function adds samples to the current voice by their file names and bytes.
-
-        Args:
-            samples (dict[str, bytes]): A dictionary of sample audio file names and their respective bytes.
-
-        Returns:
-            None
-
-        Raises:
-            ValueError: If no samples are provided.
-
-        """
-        if len(samples.keys()) == 0:
-            raise ValueError("Please add at least one sample!")
-
-        payload = {"name":self.get_name()}
-        files = list()
-        for fileName, fileBytes in samples.items():
-            files.append(("files", (fileName, io.BytesIO(fileBytes))))
-
-        _api_multipart("/voices/" + self._voiceID + "/edit", self._linkedUser.headers, data=payload, filesData=files)
-
-    def delete_voice(self):
-        """
-        This function deletes the current voice.
-
-        Returns:
-            None
-
-        Raises:
-            RuntimeError: If the voice is a premade voice.
-
-        """
-        if self._category == "premade":
-            raise RuntimeError("Cannot delete premade voices!")
-        response = _api_del("/voices/" + self._voiceID, self._linkedUser.headers)
-        self._voiceID = ""
\ No newline at end of file
