@@ -1,7 +1,7 @@
 import io
 import logging
 import threading
-from typing import Optional, BinaryIO, Callable
+from typing import Optional, BinaryIO, Callable, Union
 
 import sounddevice as sd
 import soundfile
@@ -33,7 +33,7 @@ def _api_call(requestType, path, headers, jsonData=None, filesData=None) -> requ
         response.raise_for_status()
         return response
     except requests.exceptions.RequestException as e:
-        _pretty_print_POST(response.request)
+        _pretty_print_POST(response)
         raise e
 
 def _api_get(path, headers) -> requests.Response:
@@ -48,23 +48,33 @@ def _api_json(path, headers, jsonData) -> requests.Response:
 def _api_multipart(path, headers, data=None, filesData=None):
     return _api_call("multipart", path, headers, data, filesData)
 
-def _pretty_print_POST(req):
-    logging.error('REQUEST THAT CAUSED THE ERROR:\n{}\n{}\r\n{}\r\n\r\n{}'.format(
+def _pretty_print_POST(res:requests.Response):
+    req = res.request
+    logging.debug(f"RESPONSE DATA: {res.text}")
+    logging.debug('REQUEST THAT CAUSED THE ERROR:\n{}\n{}\r\n{}\r\n\r\n{}'.format(
         '-----------START-----------',
         req.method + ' ' + req.url,
         '\r\n'.join('{}: {}'.format(k, v) for k, v in req.headers.items()),
         req.body,
     ))
 
+
+
+
 def play_audio_bytes(audioData:bytes, playInBackground:bool, portaudioDeviceID:Optional[int] = None,
                      onPlaybackStart:Callable=lambda: None, onPlaybackEnd:Callable=lambda: None) -> None:
     """
-    :param onPlaybackStart: Function to call once the playback begins
-    :param onPlaybackEnd: Function to call once the playback ends
-    :param audioData: The audio to play
-    :param playInBackground: Whether to play it in the background
-    :param portaudioDeviceID: The ID of the portaudioDevice to play it back on (Optional)
-    :return:
+    Plays the given audio and calls the given functions.
+    
+    Parameters:
+         onPlaybackStart: Function to call once the playback begins
+         onPlaybackEnd: Function to call once the playback ends
+         audioData: The audio to play
+         playInBackground: Whether to play it in the background
+         portaudioDeviceID: The ID of the portaudioDevice to play it back on (Optional)
+
+    Returns:
+        None
     """
 
     if portaudioDeviceID is None:
@@ -78,28 +88,24 @@ def play_audio_bytes(audioData:bytes, playInBackground:bool, portaudioDeviceID:O
     else:
         playbackWrapper.stream.start()
 
-def save_bytes_to_path(filepath:str, audioData:bytes) -> None:
+def save_audio_bytes(audioData:bytes, saveLocation:Union[BinaryIO,str], outputFormat) -> None:
     """
-    This function saves the audio data to the specified location.
-    soundfile is used for the conversion, so it supports any format it does.
-    :param filepath: The path where the data will be saved to.
-    :param audioData: The audio data.
-    """
-    fp = open(filepath, "wb")
-    tempSoundFile = soundfile.SoundFile(io.BytesIO(audioData))
-    sf.write(fp,tempSoundFile.read(), tempSoundFile.samplerate)
+        This function saves the audio data to the specified location OR file-like object.
+        soundfile is used for the conversion, so it supports any format it does.
 
-def save_bytes_to_file_object(fp:BinaryIO, audioData:bytes, outputFormat="mp3") -> None:
-    """
-    This function saves the audio data to the specified file like object, in the specified format.
-    soundfile is used for the conversion, so it supports any format it does.
-    :param fp: The file-like object the data will be saved to.
-    :param audioData: The audio data.
-    :param outputFormat: The output format (mp3 by default).
-    """
+        Parameters:
+            audioData: The audio data.
+            saveLocation: The path (or file-like object) where the data will be saved.
+            outputFormat: The format in which the audio will be saved
+        """
     tempSoundFile = soundfile.SoundFile(io.BytesIO(audioData))
-    sf.write(fp,tempSoundFile.read(), tempSoundFile.samplerate, format=outputFormat)
 
+    if isinstance(saveLocation, str):
+        with open(saveLocation, "wb") as fp:
+            sf.write(fp, tempSoundFile.read(), tempSoundFile.samplerate, format=outputFormat)
+    else:
+        sf.write(saveLocation, tempSoundFile.read(), tempSoundFile.samplerate, format=outputFormat)
+        saveLocation.flush()
 
 #This class just helps with the callback stuff.
 class _SDPlaybackWrapper:
